<?php
declare(strict_types=1);

namespace Inti\Test\Constraint\Provider;

use Inti\Provider\Service\ProviderManagerInterface;
use Inti\Provider\Type\TypeInterface;
use PHPUnit\Framework\Constraint\Constraint;
use function get_class;

/**
 * Class CanProvide
 *
 * @package Inti\Test\Constraint\Provider
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
final class CanProvide extends Constraint
{
    private TypeInterface $type;

    public function __construct(TypeInterface $type)
    {
        $this->type = $type;
    }

    /**
     * @param ProviderManagerInterface|array $other
     * @return bool
     */
    protected function matches($other): bool
    {
        return $other->canProvide($this->type);
    }

    protected function failureDescription($other): string
    {
        return 'the given provider ' . $this->toString();
    }

    public function toString(): string
    {
        return sprintf("can provide the type '%s'", get_class($this->type));
    }
}
