<?php
declare(strict_types=1);

namespace Inti\Test\TestCase;

use Inti\App\ConfigProvider;
use Inti\Test\Emitter\TestEmitter;
use Laminas\Diactoros\ServerRequestFactory;
use Laminas\HttpHandlerRunner\Emitter\EmitterInterface;
use Laminas\Stdlib\ArrayUtils;
use Mezzio\Application;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;

/**
 * Class ApplicationTestCase
 *
 * @package Inti\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
abstract class ApplicationTestCase extends ContainerTestCase
{
    protected array $providers = [
        ConfigProvider::class,
    ];

    protected array $config = [
        'dependencies' => [
            'invokables' => [
                EmitterInterface::class => TestEmitter::class,
            ],
        ],
    ];

    protected function createApplication(array $config = []): Application
    {
        $container = $this->createContainer($config);
        return $container->get(Application::class);
    }

    protected function runApplication(
        array $config = [],
        string $path = '/',
        string $method = 'GET'
    ): ResponseInterface
    {
        /** @noinspection PhpExpressionResultUnusedInspection */
        @list($path, $query) = explode('?', $path, 2);

        $_SERVER = ArrayUtils::merge($_SERVER, [
            'HTTP_HOST' => $this->hostname ?? 'inti.dev',
            'QUERY_STRING' => $query ?? '',
            'REQUEST_METHOD' => $method,
            'REQUEST_URI' => $path,
        ]);

        foreach (array_filter(explode('&', $query ?? '')) as $param) {
            /** @noinspection PhpExpressionResultUnusedInspection */
            @list($name, $value) = explode('=', $param, 2);
            $_GET[$name] = $value ?? '';
        }

        $request = ServerRequestFactory::fromGlobals();

        $config['dependencies']['services'][ServerRequestInterface::class] = fn() => $request;

        $application = $this->createApplication($config);
        $application->run();

        /** @var TestEmitter $emitter */
        $emitter = $this->_container->get(EmitterInterface::class);
        return $emitter->getReponse();
    }
}
