<?php
declare(strict_types=1);

namespace Inti\Test\TestCase;

use Inti\Test\Constraint\Config\HasValue;
use Laminas\ConfigAggregator\ArrayProvider;
use Laminas\ConfigAggregator\ConfigAggregator;
use Laminas\Stdlib\ArrayUtils;
use PHPUnit\Framework\Assert;
use PHPUnit\Framework\TestCase;
use ReflectionClass;

use function array_unshift;

/**
 * Class ConfigTestCase
 *
 * @package Inti\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
abstract class ConfigTestCase extends TestCase
{
    protected array $providers = [];
    protected array $config    = [];

    protected array $_config;

    protected function createConfig(array $config = []): array
    {
        $providers = $this->providers;
        $configs[] = new ArrayProvider($this->config);

        $current = new ReflectionClass($this);

        while ($current = $current->getParentClass()) {
            $props = $current->getDefaultProperties();

            if (isset($props['providers']) && !empty($props['providers'])) {
                $providers = ArrayUtils::merge($props['providers'], $providers);
            }

            if (isset($props['config']) && !empty($props['config'])) {
                array_unshift($configs, new ArrayProvider($props['config']));
            }
        }

        if (!empty($config)) {
            $configs[] = new ArrayProvider($config);
        }

        $providers = ArrayUtils::merge($providers, $configs);

        $aggregator = new ConfigAggregator($providers);
        $this->_config = $aggregator->getMergedConfig();

        return $this->_config;
    }

    /**
     * Provjerava da li data vrijednost postoji u datoj konfiguraciji.
     *
     * @param string $key
     * @param string $message
     */
    protected function assertConfigValueExists(string $key, string $message = ''): void
    {
        $constraint = new HasValue($key);
        Assert::assertThat($this->_config ?? $this->createConfig(), $constraint, $message);
    }
}
