<?php
declare(strict_types=1);

namespace Inti\Test\TestCase;

use Inti\Test\Constraint\Container\HasService;
use Inti\Test\Constraint\Container\ServiceInstanceOf;
use Laminas\ServiceManager\ServiceManager;
use PHPUnit\Framework\Assert;
use Psr\Container\ContainerInterface;

/**
 * Class ContainerTestCase
 *
 * @package Inti\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
abstract class ContainerTestCase extends ConfigTestCase
{
    protected ContainerInterface $_container;

    protected function setUp(): void
    {
        unset($this->_container);
    }

    protected function createContainer(array $config = []): ContainerInterface
    {
        $config = $this->createConfig($config);

        $this->_container = new ServiceManager($config['dependencies'] ?? []);
        $this->_container->setService('config', $config);

        return $this->_container;
    }

    /**
     * Provjerava da li je traženi servis registriran u containeru.
     *
     * @param string $service
     * @param string $message
     */
    public function assertContainerHasService(string $service, string $message = ''): void
    {
        $hasService = new HasService($service);
        Assert::assertThat($this->_container ?? $this->createContainer(), $hasService, $message);
    }

    public function assertServiceInstanceOf(string $service, string $instanceOf, string $message = ''): void
    {
        $this->assertContainerHasService($service);

        $serviceInstanceOf = new ServiceInstanceOf($service, $instanceOf);
        Assert::assertThat($this->_container ?? $this->createContainer(), $serviceInstanceOf, $message);
    }
}
