<?php
declare(strict_types=1);

namespace IntiTest\Test\TestCase;

use Inti\Test\TestCase\ConfigTestCase;
use IntiTest\Test\Mock;
use PHPUnit\Framework\ExpectationFailedException;

/**
 * Class ConfigTest
 *
 * @package IntiTest\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
final class ConfigTest extends ConfigTestCase
{
    protected array $providers = [
        Mock\ConfigProvider::class,
    ];

    protected array $config = [
        'some_config' => [
            'some_key' => 'some value',
        ],
    ];

    public function testWillCollectProvidersFromAllClasses()
    {
        $config = (new Mock\GrandChildTestCase())->createConfig();

        $this->assertArrayHasKey(Mock\BaseConfigProvider::class, $config);
        $this->assertArrayHasKey(Mock\ChildConfigProvider::class, $config);
        $this->assertArrayHasKey(Mock\GrandChildConfigProvider::class, $config);
    }

    public function testWillCollectConfigFromAllClasses()
    {
        $testCase = new Mock\GrandChildTestCase();
        $config = $testCase->createConfig();

        $testCase->assertConfigValueExists('values');
        $testCase->assertContains('base_value', $config['values']);
        $testCase->assertContains('child_value', $config['values']);
        $testCase->assertContains('grandchild_value', $config['values']);
    }

    public function testCanCheckIfKeyDoesntExists()
    {
        $this->expectException(ExpectationFailedException::class);
        $this->assertConfigValueExists('dependencies/services/config');
    }

    public function testCanCheckIfKeyExists()
    {
        $this->assertConfigValueExists('some_config/some_key');
    }

    public function testCanReadConfigFromProviders()
    {
        $config = $this->createConfig();

        $this->assertConfigValueExists('dependencies/aliases/mock_config_provider');
        $this->assertEquals(Mock\ConfigProvider::class, $config['dependencies']['aliases']['mock_config_provider']);
    }

    public function testWillUseDefaultConfig()
    {
        $config = $this->createConfig();

        $this->assertConfigValueExists('some_config/some_key');
        $this->assertEquals('some value', $config['some_config']['some_key']);
    }

    public function testCanOverrideDefaultConfig()
    {
        $config = $this->createConfig([
            'some_config' => [
                'some_key' => 'other value',
            ],
        ]);

        $this->assertConfigValueExists('some_config/some_key');
        $this->assertEquals('other value', $config['some_config']['some_key']);
    }
}
