<?php
/** @noinspection PhpFullyQualifiedNameUsageInspection */
declare(strict_types=1);

namespace IntiTest\Test\TestCase;

use Inti\Test\TestCase\ContainerTestCase;
use PHPUnit\Framework\ExpectationFailedException;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\UriFactoryInterface;

/**
 * Class ContainerTest
 *
 * @package IntiTest\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
final class ContainerTest extends ContainerTestCase
{
    protected array $providers = [
        \Laminas\Diactoros\ConfigProvider::class,
    ];

    public function testCanCheckIfServiceIsNotRegistered()
    {
        $this->expectException(ExpectationFailedException::class);
        $this->assertContainerHasService(self::class);
    }

    public function testCanCheckIfServiceIsRegistered()
    {
        $this->assertContainerHasService(UriFactoryInterface::class);
    }

    public function testCanAssertServiceIsNotOfInstance()
    {
        $this->expectException(ExpectationFailedException::class);
        $this->assertServiceInstanceOf(UriFactoryInterface::class, RequestInterface::class);
    }

    public function testCanAssertServiceIsOfInstance()
    {
        $this->assertServiceInstanceOf(UriFactoryInterface::class, UriFactoryInterface::class);
    }
}
