<?php
declare(strict_types=1);

namespace IntiTest\Test\TestCase;

use Inti\Test\TestCase\FilesystemTestTrait;
use org\bovigo\vfs\vfsStream;
use PHPUnit\Framework\ExpectationFailedException;
use PHPUnit\Framework\TestCase;

use function chmod;
use function file_put_contents;
use function mkdir;

final class FilesystemTest extends TestCase
{
    use FilesystemTestTrait;

    private string $dir;
    private string $file;

    protected function setUp(): void
    {
        vfsStream::setup();

        $this->dir = vfsStream::url('root') . '/dir';
        mkdir($this->dir, 0750);

        $this->file = $this->dir . '/file';
        file_put_contents($this->file, '');
        chmod($this->file, 0640);
    }

    public function testCanCompareFilePermissions()
    {
        $this->assertFilePermissions(0640, $this->file);
    }

    public function testCanCompareDirectoryPermissions()
    {
        $this->assertFilePermissions(0750, $this->dir);
    }

    public function testWillFailForWrongFilePermissions()
    {
        $this->expectException(ExpectationFailedException::class);
        $this->assertFilePermissions(0647, $this->file);
    }


}
