<?php
/** @noinspection PhpFullyQualifiedNameUsageInspection */
declare(strict_types=1);

namespace IntiTest\Test\TestCase;

use Inti\Provider\Type\TypeInterface;
use Inti\Test\TestCase\ContainerTestCase;
use Inti\Test\TestCase\ProviderTestTrait;
use IntiTest\Test\Mock;
use PHPUnit\Framework\ExpectationFailedException;
use Prophecy\PhpUnit\ProphecyTrait;

final class ProviderTest extends ContainerTestCase
{
    use ProviderTestTrait;
    use ProphecyTrait;

    protected array $providers = [
        \Inti\Provider\ConfigProvider::class,
    ];

    public function testWillAssertSupportedType()
    {
        $this->assertProviderCanProvide(Mock\Type::instance(), new Mock\TypeProvider());
    }

    public function testWillFailForUnsupportedType()
    {
        /** @var TypeInterface $type */
        $type = $this->prophesize(TypeInterface::class)->reveal();

        $this->expectException(ExpectationFailedException::class);
        $this->assertProviderCanProvide($type, new Mock\TypeProvider());
    }
}
